{
********************						***********************

	Player PRO 4.5x -- MAD Music Driver Definition -

	Library Version 4.5

	To use with MADH Library for CodeWarrior

	Antoine ROSSET
	16 Tranchees
	1206 GENEVA
	SWITZERLAND

	Thank you for your interest in PlayerPRO !

	FAX:			(+41 22) 346 11 97
	PHONE: 		(+41 79) 203 74 62
	Internet: rosset@dial.eunet.ch


	Ported to Pascal by Federico Filipponi (March 1997)
	e-mail:	fedefil@fub.it
					fedefil@kagi.com
	
	Based on a previous work by Matthias Wuttke
	e-mail: wuttke@stein.teuto.de
					matthias@kagi.com

********************						***********************
}

unit RDriver;

interface

{$ALIGN MAC68K}

	uses
		Files, Retrace, Sound, Types, 
		MAD;

{
********************						***********************
*** 			   		 Driver structure definition (constants)		***
********************						***********************
}
	const
		NUMBER_NOTES = 96;
		NOFINETUNE = 8363;
		MIN_VOLUME = 0;
		MAX_VOLUME = 64;
		MAXTRACK = 32;		
		MAXINSTRU = 64;
		MAXSAMPLE = 25;
		MAXPATTERN = 200;
		MAXPATTERNSIZE = 900;
		MAX_ARP = 3;
		MAXPLUG = 40;
		
{
********************						***********************
*** 			  		  Error messages 						***
********************						***********************
}
		MADNeedMemory = -1;
		MADReadingErr	= -2;
		MADIncompatibleFile = -3;
		MADLibraryNotInitialized = -4;
		MADParametersErr = -5;
		MADUnknowErr = -6;
		MADSoundManagerErr = -7;
		MADOrderNotImplemented = -8;
		MADFileNotSupportedByThisPlug	= -9;
		MADCannotFindPlug = -10;
		
{
********************						***********************
*** 			     Driver Settings definition	(constants)				***
********************						***********************
}
		ASCSoundDriver = 1;				{ MAC ONLY }
		AWACSoundDriver = 2;			{ MAC ONLY }
		MIDISoundDriver = 3;			{ MAC ONLY }
		SoundManagerDriver = 4;		{ MAC ONLY You should use only SoundManagerDriver for full compatibility ! }
		QK25SoundDriver = 5;			{ MAC ONLY }
		DigiDesignSoundDriver = 6;{ MAC ONLY }
		BeOSSoundDriver = 7;			{ BE ONLY when using with BeOS compatible systems ! - NOT FOR MAC }
		DirectSound95NT = 8;			{ WINDOWS 95/NT ONLY when using with PC compatible systems ! - NOT FOR MAC }
		Wave95NT = 9;							{ WINDOWS 95/NT ONLY when using with PC compatible systems ! - NOT FOR MAC }
		NoHardwareDriver = 10;		{ NO HARDWARE CONNECTION, will not produce any sound }

		MonoOutPut = 1;
		StereoOutPut = 2;
		DeluxeStereoOutPut = 3;
		PolyPhonic = 4;						{ Do NOT use it ! Standard hardware doesn't support it ! }

{
********************						***********************
*** 					   EFFECTS ID							***
********************						***********************
}
		arpeggioE = 0;
		downslideE = 1;
		upslideE = 2;
		portamentoE = 3;
		vibratoE = 4;
		portaslideE = 5;
		vibratoslideE = 6;
		nothingE = 7;
		offsetE = 9;
		slidevolE = 10;
		fastskipE = 11;
		volumeE = 12;
		skipE = 13;
		extendedE = 14;
		speedE = 15;

{
********************						***********************
*** 			   Channel structure definition					***
********************						***********************
}
	type
		Channel = record
			ID: LongInt;					{ Channel ID - 0 to 31 }

			begPtr: Ptr;				  { Sample Data Ptr - Beginning of data }
			maxPtr: Ptr;					{ Sample Data Ptr - End of data }
			curPtr: Ptr;					{ Sample Data Ptr - Current position }
			sizePtr: LongInt;			{ Sample Size in bytes }
			
			amp: LongInt;					{ Sample amplitude: 8 or 16 bits }
			
			loopBeg: LongInt;			{ Loop Beginning }
			loopSize: LongInt;		{ Loop Size }
			
			ins: LongInt;					{ Current Instrument ID }
			insOld: LongInt;			{ Previous Instrument ID played on this channel }
			samp: LongInt;				{ Current Sample ID }
			
			fineTune: LongInt;		{ Finetune }
			
			note: LongInt;				{ Note }
			noteOld: LongInt;			{ Previous note played on this channel }
			
			period: LongInt;			{ Current period }
			periodOld: LongInt;		{ Previous period played on this channel }
			
			vol: LongInt;					{ Channel vol (0 to 64) }
			cmd: LongInt;					{ Command }
			arg: Byte;						{ Argument of command }
			volcmd: Byte;					{ Volume Command }
			
			arp: array [0..(MAX_ARP - 1)] of LongInt;		{ Used for arpeggio command }
			arpindex: LongInt;		{ Used for arpeggio command }
			
			viboffset: LongInt;		{ Used for vibrato command }
			vibdepth: LongInt;		{ Used for vibrato command }
			vibrate: LongInt;			{ Used for vibrato command }
			
			slide: LongInt;				{ Used for slideUp and slideDown command }
			
			pitchgoal: LongInt;		{ Used for portamento command }
			pitchrate: LongInt;		{ Used for portamento command }
			
			volumerate: LongInt;	{ Used for slideVolume command }
			
			oldArg: array [0..15] of LongInt;

			samplePtr: Ptr;				{ Used internaly by MADPlaySoundData & MADPlaySndHandle }
			
			{}
			
			KeyOn: Boolean;
			a: Integer;
			b: Integer;
			p: Integer;
			volEnv: LongInt;
			volFade: LongInt;
			
			lAC: LongInt;
			
			prevPtr: Ptr;
			lastWord, curLastWord: LongInt;
			curLevel: LongInt;
			
			prevVol, curPrevVol: LongInt;
			
			{}
			
			GEffect: Boolean;
			GPat, GReader: Integer;
		end;
	ChannelPtr = ^Channel;

{
********************						***********************
*** 		Music description - used in Import/Export filter	***
********************						***********************
}
	MADPartition = record
		header: MADSpecPtr;		{ Music Header - See 'MAD.p' }
		partition: array [0..(MAXPATTERN - 1)] of PatDataPtr;		{ Patterns }
		sample: array [0..(MAXINSTRU - 1), 0..(MAXSAMPLE - 1)] of sDataPtr;		{ Instruments }	
	end;
	MADPartitionPtr = ^MADPartition;

{
********************						***********************
*** 			     Driver Settings definition				***
********************						***********************
}
	MADDriverSettings = record
		numChn: Integer;								{ Active tracks from 2 to 32, automatically setup when a new music is loaded }
		outPutBits: Integer;						{ 8 or 16 Bits }
		outPutRate: UnsignedLong;				{ Fixed number, by example : rate44khz, rate22050hz, rate22khz, rate11khz, rate11025hz }
		outPutMode: Integer;						{ MonoOutPut, StereoOutPut or DeluxeStereoOutPut ? }
		driverMode: Integer;						{ ASCSoundDriver, AWACSoundDriver, MIDISoundDriver or SoundManagerDriver }
		antiAliasing: Boolean;					{ Use AntiAliasing filter ? true/false }
		repeatMusic: Boolean;						{ If music finished, repeat it or stop. }
		sysMemory: Boolean;							{ Allocate memory in Application Heap (false) or in System Heap (true) }
		Interpolation: Boolean;					{ Sound Interpolation active? true/false }
		MicroDelay: Boolean;						{ Micro delay active? Used only in DeluxeStereoOutPut outPutMode. }
		MicroDelaySize: LongInt;				{ Micro delay duration (in ms, max 1 sec = 1000 ms) }
		surround: Boolean;							{ Surround effect active? true/false }
		Reverb: Boolean;								{ Reverb effect active? true/false }
		ReverbSize: LongInt;						{ Reverb delay duration (in ms, min = 25 ms, max 1 sec = 1000 ms) }
		ReverbStrength: LongInt;				{ Reverb strength in % (0 <-> 70) }
		TickRemover: Boolean;						{ Remove volume/sample/loop ticks. }
	end;
	MADDriverSettingsPtr = ^MADDriverSettings;

{
******************************************************************
******************* MUSICS IMPORT/EXPORT PLUGS  *****************

 	Protocol Version 2.3
 
 	To use with PlayerPRO for CodeWarrior
 
	Your main function have to be in this form:
	OSErr main( 	OSType order,
					Ptr AlienFileName,
					MADPartition *MadFile,
					PPInfoRec *info,
					MADDriverSettings *DriverParam);

	Actual plug have to support these orders:

	order: 'TEST':	check the AlienFile to see if your Plug really supports it.
	order: 'IMPT':	convert the AlienFile into a MADPartition. You have to allocate MADPartition.
	order: 'INFO':	Fill PPInfoRec structure.
	order: 'EXPT':	Convert the MADPartition into AlienFile. You have to create the AlienFile.
					Don't delete the MADPartition Structure after conversion !!

	An IMPORT plug have to support these orders: 'TEST', 'IMPT', 'INFO'
	An EXPORT plug have to support these orders: 'EXPT'
 	An IMPORT/EXPORT plug have to support these orders: 'TEST', 'IMPT', 'INFO', 'EXPT'

	About Resources:

	Your Plug should have: Creator: 'SNPL', Type: 'IMPL' - MAC ONLY

	Your Plug have to have these resources - MAC ONLY:

	- One segment CODE 1000 with 68k Code
	- One segment PPCC 1000 with PPC Code (OPTIONAL: if PlayerPRO in PPC cannot find it, it will use the CODE 1000 resource)
	- One STR# resource :

		1 string: which kind of files your plug support (OSType value!!! 4 char) By example: 'STrk', '669 ', etc...
		2 string: what does your Plug: EXPL : only Export files, IMPL : only Import Files, EXIM : import AND export.
		3 string: string that will be used in Import and Export menu of PlayerPRO
		4 string: Copyright string of this plug.
 
********************						***********************
}
	PPInfoRec = record
		internalFileName: packed array [0..59] of Char;
		formatDescription: packed array [0..59] of Char;
		
		totalPatterns: LongInt;
		partitionLength: LongInt;
		
		totalTracks: Integer;
		totalInstruments: Integer;
		
		signature: OSType;
		
		fileSize: LongInt;
	end;
	PPInfoRecPtr = ^PPInfoRec;

{
********************						***********************
*** 			Informations about Plugs: ThePlug[]				***
********************						***********************
}
	PlugInfo = record
		IOPlug: Handle;												{ Plug CODE }
		MenuName: Str63;											{ Plug name }
		AuthorString: Str63;									{ Plug author }
		theFile: FSSpec;											{ Location of plug file }
		theType: packed array [0..4] of Char;	{ OSType of file support }
		mode: OSType;													{ Mode support : Import +/ Export }
		hasPPCCode: Boolean;									{ Is Plug FAT? }
	end;
	PlugInfoPtr = ^PlugInfo;
	
{
********************						***********************
*** 		Global structure : PlayerPRO variables				***
********************						***********************
}
	MADDriverRec = record
		IDType: LongInt;																		{ IDType = 'MADD' -- READ ONLY -- }
{
		**********************
		** Public variables **
		**********************
}
		DriverSettings: MADDriverSettings;									{ Driver SetUp -- READ ONLY -- }

		{**  Current music in memory, loaded with RLoadMusic() by example **}

		header: MADSpecPtr;		{ MAD Header  -- READ ONLY -- }
		partition: array [0..(MAXPATTERN - 1)] of PatDataPtr;		{ Patterns    -- READ ONLY -- }
		sample: array [0..(MAXINSTRU - 1), 0..(MAXSAMPLE - 1)] of sDataPtr;		{ Instruments -- READ ONLY -- }	
		
		{**  Drivers variables **}
		
		chan: array [0..(MAXTRACK - 1)] of Channel;					{ Current driver channels -- READ ONLY -- }
		musicEnd: Boolean;																	{ Is music playing finished? -- READ ONLY -- }
		Tube: array [0..(MAXTRACK - 1)] of Integer;					{ Used in 'Tracks View' Window - View menu  }
		PartitionReader: Integer;														{ Current position in pattern (0...64) }
		Pat: Integer;																				{ Current ID Pattern, see 'Patterns list' }
		PL: Integer;																				{ Current position in partition, see 'Partition list' }
		VolExt: array [0..(MAXTRACK - 1)] of Integer;				{ was long! Volumes settings for each track, see 'Adaptators' window. from 0 to 64 }
		VolGlobal: LongInt;																	{ Global SOFTWARE volume (This is NOT Mac hardware volume!) from 0 to 64 }
		speed: Integer;																			{ Current speed, see speed Effect }
		finespeed: Integer;																	{ Current finespeed, see speed Effect }
		InstruTube: array [0..(MAXINSTRU - 1)] of Integer;	{ Used in 'Instrument View' Window - View menu }
		VExt: Integer;																			{ External music speed, see 'Adaptators' window. 80 = normal }
		FreqExt: Integer;																		{ External music pitch, see 'Adaptators' window. 80 = normal }
		Reading: Boolean;																		{ Reading indicator }
		LeftRight: array [0..(MAXTRACK - 1)] of Integer;		{ Left/Right % for Deluxe Driver }
		MusicChannelPP: SndChannelPtr;											{ MAC ONLY The SndChannelPtr to apply SndDoCommand, etc. }
																												{ ONLY available if you are using SoundManager driver }

		{** Plugs Import/Export variables **}

		ThePlug: PlugInfoPtr;																{ Pointer to Plugs code & infos }
		TotalPlug: Integer;																	{ no. of Plugs in pointer ThePlug }

		{** Private variables - Not documented **}
		{* DO NOT MODIFY OR USE these variables *}

		MIN_PITCH, MAX_PITCH: LongInt;
		smallcounter, trackDiv: Integer;
		FREQBASE: LongInt;
		InstruActif: array [0..(MAXINSTRU - 1)] of Integer;
		SysHeapPtr, Vol, IntDataPtr, OscilloWavePtr: Ptr;
		JumpToNextPattern, endPattern, MADPlay: Boolean;
		ASCBUFFER: LongInt;
		BufSize: LongInt;
		VSYNC, BufCounter, BytesToGenerate: LongInt;
		vibrato_table: array [0..63] of Integer;
		MIDIPortRefNum, gOutNodeRefNum: Integer;
		InstuNoOld: array [0..(MAXTRACK - 1)] of Integer;
		NoteOld: array [0..(MAXTRACK - 1)] of Integer;
		VelocityOld: array [0..(MAXTRACK - 1)] of Integer;
		TrackLineReading: array [0..(MAXTRACK - 1)] of Boolean;
		OverShoot: Ptr;
		DASCBuffer: LongIntPtr;
		DASCBuffer8: IntegerPtr;
		VBLA5: LongInt;
		mytab: array [0..11] of LongInt;
		MDelay: LongInt;
		RDelay: LongInt;
		ReverbPtr: Ptr;
		TheHeader: SndDoubleBufferHeader;		{ MAC ONLY }
		pseudoChanAWAC: SndChannelPtr;			{ MAC ONLY }
		VBL: VBLTask;												{ MAC ONLY }
	end;
	MADDriverRecPtr = ^MADDriverRec;


{
******************************************************************
***************** MUSIC RESOURCE TYPE IDENTIFIER *****************

	The type MADResType is used instead of ResType in the 
	MADLoadMusicRsrc function.
	This because, when generating 68k code, the type ResType, which
	corresponds to a PACKED ARRAY [1..4] OF CHAR, is ALWAYS passed
	as a pointer (It's a C function, after all).
	To have it passed the right way, it must be a 4-bytes long type,
	like an UnsignedLong. When calling MADLoadMusicRsrc, the ResType 
	parameter must be passed as MADResType, like in the following example:
	
	rsrcType := MADResType(rsrcType); (where rsrcType is of type ResType)
	err := MADLoadMusicRsrc(rsrcType, 1);
	
	This affects 68k code only. When generating PPC code, ResType can
	be used normally.

******************************************************************
}
{$IFC GENERATING68K}
	MADResType = UnsignedLong;
{$ELSEC}
	MADResType = ResType;
{$ENDC}

{
******************************************************************
********************** PLUG TYPE IDENTIFIER **********************

	The type MADPlugTypePtr is used instead of OSTypePtr in many of 
	the following functions (for compatibility with platforms other 
	than the Mac OS).
	The type conversion from OSType (4 bytes) to MADPlugType (5 bytes) 
	is obtained with a call to the OSType2MADPlugType procedure (see 
	the file Example.p for an example of its use).
	As an alternative, you can obtain the plug type directly in the 
	right format by calling, for example, the MADMusicIdentifyFSp 
	function.

******************************************************************
}
MADPlugType = packed array [0..4] of Char;
MADPlugTypePtr = ^MADPlugType;

{
******************************************************************
*************** 					  FUNCTIONS							****************
******************************************************************
}

	function MADGetMADDriverPtr: MADDriverRecPtr;		{ Get MADDriver structure pointer. }
	C; EXTERNAL;
	
	function MADInitLibrary(PlugsFolderName: CStringPtr; SysMemory: Boolean): OSErr;	{ Library initialisation, you have to CALL this function if you want to use other functions & variables }
 	C; EXTERNAL;
	function MADDisposeLibrary: OSErr;							{ Close Library, close music, close driver, free all memory }
 	C; EXTERNAL;

	procedure MADGetBestDriver(DriverInitParam: MADDriverSettingsPtr);	{ Found and identify the current Mac sound hardware and fill DriverInitParam }
 	C; EXTERNAL;
	function MADCreateDriver(DriverInitParam: MADDriverSettingsPtr): OSErr;		{ Music Driver initialization and memory allocation }
 	C; EXTERNAL;
	function MADDisposeDriver: OSErr;								{ Dispose the music driver, use it after RInitMusic() }
 	C; EXTERNAL;

	function MADStartDriver: OSErr;									{ NEW - Activate the sound generating procedure (interruption) }
 	C; EXTERNAL;
	function MADStopDriver: OSErr;									{ NEW - DESActivate the sound generating procedure (interruption) }
 	C; EXTERNAL;

	function MADPlayMusic: OSErr;										{ NEW - Read and play current music in memory - Call MADStartInterruption BEFORE }
 	C; EXTERNAL;
	function MADStopMusic: OSErr;										{ NEW - Stop reading current music in memory }
 	C; EXTERNAL;

	function MADReset: OSErr;												{ Reset the current music at the start position }
 	C; EXTERNAL;
	function MADGetMusicStatus(var fullTime, curTime: LongInt): OSErr;	{ Get informations about music position and duration }
 	C; EXTERNAL;

	function MADSetHardwareVolume(theVol: LongInt): OSErr;		{ 0...64, Mac HARDWARE volume, see MADDriver->VolGlobal for SOFTWARE volume }
 	C; EXTERNAL;
	function MADGetHardwareVolume: LongInt;										{ Return HARDWARE volume, see MADDriver->VolGlobal for SOFTWARE volume }
 	C; EXTERNAL;

	function MADLoadMusicRsrc(IDName: MADResType; IDNo: Integer): OSErr;	{ Load a MAD Rsrc into memory }
 	C; EXTERNAL;
	function MADLoadMusicPtr(myPtr: Ptr): OSErr;							{ Load a MAD Ptr into memory, you can DisposPtr your Ptr after this call }
 	C; EXTERNAL;
	function MADLoadMusicPartition(aPartition: MADPartitionPtr): OSErr;		{ Load a MAD partition into memory }
 	C; EXTERNAL;

	function MADLoadMusicFilePString(plugType: MADPlugTypePtr; fName: Str255): OSErr;			{ Load a music file with plug }
 	C; EXTERNAL;
	function MADLoadMusicFileCString(plugType: MADPlugTypePtr; fName: CStringPtr): OSErr;	{ Load a music file with plug }
 	C; EXTERNAL;
	function MADLoadMusicFSpFile(plugType: MADPlugTypePtr; theSpec: FSSpecPtr): OSErr;		{ Load a music file with plug }
 	C; EXTERNAL;

	function MADMusicIdentifyPString(plugType: MADPlugTypePtr; pName: Str255): OSErr;			{ Identify what kind of music format is pName file. }
 	C; EXTERNAL;
	function MADMusicIdentifyCString(plugType: MADPlugTypePtr; cName: CStringPtr): OSErr;	{ Identify what kind of music format is cName file. }
 	C; EXTERNAL;
	function MADMusicIdentifyFSp(plugType: MADPlugTypePtr; theSpec: FSSpecPtr): OSErr;		{ Same as above but with a FSSpec pointer }
 	C; EXTERNAL;

	function MADPlugAvailable(plugType: MADPlugTypePtr): Boolean;			{ Is plug 'plugType' available? }
 	C; EXTERNAL;

	function MADDisposeMusic: OSErr;								{ Dispose the current music }
 	C; EXTERNAL;

	procedure	MADChangeTracks(i: Integer);					{ Change current tracks number of the music driver }
 	C; EXTERNAL;
	procedure	MADCleanDriver(intDriver: MADDriverRecPtr);			{ Clean the driver : stop playing sounds }
 	C; EXTERNAL;

	function GetMADCommand(	position: Integer;			{ Extract a Command from a PatData structure }
													channel: Integer;
						 							aPatData: PatDataPtr): CmdPtr;
 	C; EXTERNAL;

	function MADPlaySndHandle(sound: Handle;						{ Handle to a 'snd ' handle, by ex: GetResource('snd ', 128); }
														chan: LongInt;						{ channel ID on which to play sound }
														note: LongInt): OSErr;		{ note: 0 to NUMBER_NOTES or 0xFF: play sound at his normal sampleRate Khz }
 	C; EXTERNAL;

	function MADPlaySoundData(soundPtr: Ptr;						{ Sound Pointer to data }
														size: LongInt;						{ Sound size in bytes }
														channel: LongInt;					{ channel ID on which to play sound }
														note: LongInt;						{ note: 0 to NUMBER_NOTES or 0xFF: play sound at 22 Khz }
														amplitude: LongInt;				{ 8 or 16 bits }
														loopBeg: LongInt;					{ loop beginning }
														loopSize: LongInt;				{ loop size in bytes }
														rate: UnsignedLong): OSErr;		{ sample rate of the sound data, by ex: rate22khz }
 	C; EXTERNAL;

	function MADPlaySoundDataSYNC(soundPtr: Ptr;				{ Sound Pointer to data }
																size: LongInt;				{ Sound size in bytes }
																channel: LongInt;			{ channel ID on which to play sound }
																note: LongInt;				{ note: 0 to NUMBER_NOTES or 0xFF: play sound at 22 Khz }
																amplitude: LongInt;		{ 8 or 16 bits }
																loopBeg: LongInt;			{ loop beginning }
																loopSize: LongInt;		{ loop size in bytes }
																rate: UnsignedLong): OSErr;{ sample rate of the sound data, by ex: rate22khz }
 	C; EXTERNAL;

	procedure OSType2MADPlugType(theType: OSType; var thePlug: MADPlugType);

implementation
	
{	Used to convert from OSType (4 bytes) to MADPlugType (5 bytes) }
	procedure OSType2MADPlugType(theType: OSType; var thePlug: MADPlugType);
		type
			UnsignedLongPtr = ^UnsignedLong;
	begin
		UnsignedLongPtr(@thePlug)^ := UnsignedLongPtr(@theType)^;
		thePlug[4] := Char(0);
	end;

{$ALIGN RESET}

end.